@@ -1,18 +1,21 @@
 """
 Great Assertions.
 
-This library is inspired by the Great Expectations library and has made various expectations found in Great Expectations available when using the inbuilt python unittest assertions.
-For example if you wanted to use expect_column_values_to_be_between then you can access assertExpectColumnValuesToBeBetween.
+This library is inspired by the Great Expectations library and has made various expectations
+found in Great Expectations available when using the inbuilt python unittest assertions.
 
 The library has also added in further expectations, which may be similar or new.
 """
 
 import unittest
+import json
+import sys
 from datetime import datetime
 from src.utils import (
     _get_dataframe_import_type,
     _default_null_dates,
     _get_dataframe_type,
+    _get_date_time_data,
 )
 from typing import Optional, Union, Set, List
 from unittest.runner import TextTestResult
@@ -22,6 +25,7 @@ class GreatAssertionResult(TextTestResult):
     def __init__(self, *args, **kwargs):
         super(GreatAssertionResult, self).__init__(*args, **kwargs)
         self.successes = []
+        self.extended = {}
 
     def addSuccess(self, test):
         super(GreatAssertionResult, self).addSuccess(test)
@@ -47,21 +51,20 @@ class GreatAssertionResult(TextTestResult):
             ["skipped", len(skipped)],
         ]
 
-        return pd.DataFrame(data, columns=["type", "quantity"])
+        return pd.DataFrame(data, columns=["Type", "Quantity"])
 
-    def _test_info(self, iter, status: str):
+    def __test_info(self, iter, status: str):
         return [iter[0]._testMethodName, str(iter[1]), status]
 
     def to_full_results_table(self):
         import pandas as pd
-        import numpy as np
 
         data = []
-        [data.append(self._test_info(f, "Fail")) for f in self.failures]
-        [data.append(self._test_info(e, "Error")) for e in self.errors]
-        [data.append(self._test_info(s, "Skip")) for s in self.skipped]
+        [data.append(self.__test_info(f, "Fail")) for f in self.failures]
+        [data.append(self.__test_info(e, "Error")) for e in self.errors]
+        [data.append(self.__test_info(s, "Skip")) for s in self.skipped]
         [
-            data.append([success._testMethodName, np.NaN, "Pass"])
+            data.append([success._testMethodName, "", "Pass"])
             for success in self.successes
         ]
 
@@ -69,14 +72,93 @@ class GreatAssertionResult(TextTestResult):
 
         return pd.DataFrame(data, columns=col)
 
+    @property
+    def _get_grouped_results(self):
+        return self.to_results_table().groupby(["Type"]).sum()
+
     def to_pie(self, title="Test Result", colors=["gray", "red", "blue", "green"]):
-        return (
-            self.to_results_table()
-            .groupby(["type"])
-            .sum()
-            .plot(kind="pie", y="Quantity", title=title, colors=colors)
+        return self._get_grouped_results.plot(
+            kind="pie", y="Quantity", title=title, colors=colors
         )
 
+    def to_barh(self, title="Test Result", color=["gray", "red", "blue", "green"]):
+        return self._get_grouped_results.plot(
+            kind="barh", y="Quantity", title=title, color=color
+        )
+
+    def __test_info_with_extended(self, id, date, iter, status: str):
+
+        if status == "Pass":
+            test_method = iter._testMethodName
+            information = ""
+            test_id = iter.extended["id"]
+            extended = iter.extended
+        else:
+            test_method = iter[0]._testMethodName
+            information = str(iter[1])
+            test_id = iter[0].extended["id"] if hasattr(iter[0], "extended") else -1
+            extended = iter[0].extended if hasattr(iter[0], "extended") else {}
+
+        return [
+            id,
+            date,
+            test_method,
+            information,
+            test_id,
+            status,
+            json.dumps(extended),
+        ]
+
+    @property
+    def _to_extended_info(self):
+        import pandas as pd
+
+        id, date = _get_date_time_data()
+
+        data = []
+        [
+            data.append(self.__test_info_with_extended(id, date, f, "Fail"))
+            for f in self.failures
+        ]
+        [
+            data.append(self.__test_info_with_extended(id, date, e, "Error"))
+            for e in self.errors
+        ]
+        [
+            data.append(self.__test_info_with_extended(id, date, sk, "Skip"))
+            for sk in self.skipped
+        ]
+        [
+            data.append(self.__test_info_with_extended(id, date, su, "Pass"))
+            for su in self.successes
+        ]
+
+        col = [
+            "run_id",
+            "timestamp",
+            "method",
+            "information",
+            "test_id",
+            "status",
+            "extended",
+        ]
+
+        return pd.DataFrame(data, columns=col)
+
+    def save(self, format, **args):
+        """This method will write the results to a chosen format."""
+
+        extended_info = self._to_extended_info
+
+        if format.lower() == "databricks" or format.lower() == "pyspark":
+            spark = args["spark"]
+            result_table = (
+                args["result_table"] if "result_table" in args else "ga_result"
+            )
+
+            spark_df = spark.createDataFrame(extended_info)
+            spark_df.write.mode("append").saveAsTable(result_table)
+
 
 class GreatAssertions(unittest.TestCase):
     """
@@ -86,7 +168,7 @@ class GreatAssertions(unittest.TestCase):
 
     """
 
-    def assertExpectTableRowCountToEqual(self, df, expected_count: int, msg=""):
+    def expect_table_row_count_to_equal(self, df, expected_count: int, msg=""):
         """Expect the number of rows to equal the count.
 
         Parameters
@@ -99,18 +181,204 @@ class GreatAssertions(unittest.TestCase):
         df = _get_dataframe_import_type(df)
         actual_row_count = df.row_count
 
+        self.extended = {
+            "id": 1,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "exp_count": expected_count,
+                "act_count": actual_row_count,
+            },
+        }
+
         if expected_count != actual_row_count:
             msg = self._formatMessage(
                 msg,
                 f"expected row count is {expected_count} the actual was {actual_row_count}",
             )
+
             raise self.failureException(msg)
 
         return
 
-    expect_table_row_count_to_equal = assertExpectTableRowCountToEqual
+    def expect_table_row_count_to_be_less_than(
+        self, df, expected_max_count: int, msg=""
+    ):
+        """Expect the number of rows to be less than the count.
 
-    def assertExpectColumnValuesToBeBetween(
+        Parameters
+        ----------
+            df (DataFrame)       : Pandas or PySpark DataFrame
+            expected_count (int) : The expected row count of the DataFrame
+            msg (str)            : Optional message if the assertion fails
+        """
+
+        df = _get_dataframe_import_type(df)
+        actual_row_count = df.row_count
+
+        self.extended = {
+            "id": 2,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "exp_max_count": expected_max_count,
+                "act_count": actual_row_count,
+            },
+        }
+
+        if actual_row_count >= expected_max_count:
+            msg = self._formatMessage(
+                msg,
+                f"expected row count of maximum {expected_max_count} but the actual was {actual_row_count}",
+            )
+            raise self.failureException(msg)
+
+        return
+
+    def expect_table_row_count_to_be_greater_than(
+        self, df, expected_min_count: int, msg=""
+    ):
+        """Expect the number of rows to be greater than the count.
+
+        Parameters
+        ----------
+            df (DataFrame)       : Pandas or PySpark DataFrame
+            expected_count (int) : The expected row count of the DataFrame
+            msg (str)            : Optional message if the assertion fails
+        """
+
+        df = _get_dataframe_import_type(df)
+        actual_row_count = df.row_count
+
+        self.extended = {
+            "id": 3,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "exp_min_count": expected_min_count,
+                "act_count": actual_row_count,
+            },
+        }
+
+        if actual_row_count <= expected_min_count:
+            msg = self._formatMessage(
+                msg,
+                f"expected row count of at least {expected_min_count} but the actual was {actual_row_count}",
+            )
+            raise self.failureException(msg)
+
+        return
+
+    def expect_table_has_no_duplicate_rows(self, df, msg=""):
+        """Expect the table to only have unique rows.
+
+        Parameters
+        ----------
+            df (DataFrame) : Pandas or PySpark DataFrame
+            msg (str)      : Optional message if the assertion fails
+        """
+
+        df = _get_dataframe_import_type(df)
+        actual_row_count = df.row_count
+        unique_rows = df.drop_duplicates().row_count
+
+        self.extended = {
+            "id": 4,
+            "name": sys._getframe().f_code.co_name,
+        }
+
+        if actual_row_count != unique_rows:
+            msg = self._formatMessage(
+                msg,
+                "Table contains duplicate rows",
+            )
+            raise self.failureException(msg)
+
+        return
+
+    def expect_column_has_no_duplicate_rows(self, df, columns="*", msg=""):
+        """Expect the column/s to only have unique rows.
+
+        Parameters
+        ----------
+            df (DataFrame)     : Pandas or PySpark DataFrame
+            columns (str/list) : Single, Array, Empty or * for all columns
+            msg (str)          : Optional message if the assertion fails
+        """
+
+        self.extended = {
+            "id": 5,
+            "name": sys._getframe().f_code.co_name,
+            "value": columns,
+        }
+
+        def assert_column(column, msg):
+            """Internal column assertion - uses df scope from outer method."""
+            try:
+                df_col = _get_dataframe_import_type(df).get_column(column)
+                if df_col.row_count != df_col.drop_duplicates().row_count:
+                    msg = self._formatMessage(
+                        msg,
+                        f"Column {column} contains a duplicate value",
+                    )
+                    self.extended["value"] = column
+                    raise self.failureException(msg)
+            except KeyError:
+                msg = self._formatMessage(
+                    msg,
+                    f"Column {column} is not valid",
+                )
+                raise self.failureException(msg)
+
+        if columns == "*":
+            [assert_column(column, msg) for column in list(df.columns)]
+        elif type(columns) is list:
+            [assert_column(column, msg) for column in columns]
+        elif type(columns) is str:
+            assert_column(columns, msg)
+        else:
+            msg = self._formatMessage(
+                msg,
+                "Check help for method usage",
+            )
+            raise self.failureException(msg)
+
+        return
+
+    def expect_column_value_to_equal(self, df, column: str, value: object, msg=""):
+        """
+        Expect the provided column and its value to equal.
+
+        Parameters
+        ----------
+            df (DataFrame) : Pandas or PySpark DataFrame
+            column (str)   : The name of the column to be examined
+            value (float)  : The value of the column
+            msg (str)      : Optional message if the assertion fails
+        """
+
+        df = _get_dataframe_import_type(df)
+        results = df.filter(column, value)
+
+        self.extended = {
+            "id": 6,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "expected_value": value,
+                "actual_value": results.row_count,
+            },
+        }
+
+        if results.row_count > 0:
+            first_found = results.first(column)
+            msg = self._formatMessage(
+                msg,
+                f"Column {column} was not equal, found {first_found}",
+            )
+            self.extended["values"]["first_found"] = first_found
+            raise self.failureException(msg)
+
+        return
+
+    def expect_column_values_to_be_between(
         self, df, column: str, min_value: float, max_value: float, msg=""
     ):
         """
@@ -134,12 +402,23 @@ class GreatAssertions(unittest.TestCase):
 
         df = _get_dataframe_import_type(df)
 
+        self.extended = {
+            "id": 7,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "exp_min_value": min_value,
+                "exp_max_value": max_value,
+            },
+        }
+
         column_min = df.column_min(column)
         if float(column_min) < float(min_value):
             msg = self._formatMessage(
                 msg,
                 f"Min value provided ({min_value}) must be less than column {column} value of {column_min}",
             )
+            self.extended["values"]["act_min_value"] = float(column_min)
             raise self.failureException(msg)
 
         column_max = df.column_max(column)
@@ -148,13 +427,12 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Max value provided ({max_value}) must be greater than column {column} value of {column_max}",
             )
+            self.extended["values"]["act_max_value"] = float(column_max)
             raise self.failureException(msg)
 
         return
 
-    expect_column_values_to_be_between = assertExpectColumnValuesToBeBetween
-
-    def assertExpectColumnValuesToMatchRegex(self, df, column: str, regex: str, msg=""):
+    def expect_column_values_to_match_regex(self, df, column: str, regex: str, msg=""):
         """
         Expect column entries to be strings that do NOT match a given regular expression.
 
@@ -166,89 +444,114 @@ class GreatAssertions(unittest.TestCase):
 
         df = _get_dataframe_import_type(df)
 
+        self.extended = {
+            "id": 8,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "regular_expression": regex,
+            },
+        }
+
         results = df.check_regex(column, regex)
         if results.row_count > 0:
+            first_found = results.first(column)
             msg = self._formatMessage(
                 msg,
-                f"Column {column} did not match regular expression, found {results.first(column)}",
+                f"Column {column} did not match regular expression, found {first_found}",
             )
+            self.extended["values"]["first_found"] = first_found
             raise self.failureException(msg)
 
         return
 
-    expect_column_values_to_match_regex = assertExpectColumnValuesToMatchRegex
-
-    def assertExpectColumnValuesToBeInSet(
-        self, df, column: str, value_set: set, msg=""
+    def expect_column_values_to_be_in_set(
+        self, df, column: str, value_set: set, ignore_case=False, msg=""
     ):
         """
         Expect each column value to be in a given set.
 
         Parameters
         ----------
-            df (DataFrame)  : Pandas or PySpark DataFrame
-            column (str)    : The name of the column to be examined
-            value_set (str) : Set of values found in the column
-            msg (str)       : Optional message if the assertion fails
+            df (DataFrame)     : Pandas or PySpark DataFrame
+            column (str)       : The name of the column to be examined
+            value_set (str)    : Set of values found in the column
+            ignore_case (bool) : Compare ignoring case sensitivity default False
+            msg (str)          : Optional message if the assertion fails
         """
 
-        df = _get_dataframe_type(df)
+        df = _get_dataframe_import_type(df)
 
-        results = df[~df[column].isin(value_set)].eq(False)
-        if len(results) > 0:
+        self.extended = {
+            "id": 9,
+            "name": sys._getframe().f_code.co_name,
+        }
+
+        results = df.is_in_set(column, value_set, ignore_case)
+        if results.row_count > 0:
             # Sort if possible, otherwise just output
+            column_unique_list = []
             try:
-                column_unique_list = df[column].unique().tolist()
+                column_unique_list = df.unique_list(column)
                 column_unique_list = sorted(column_unique_list)
             except TypeError:
                 pass
 
             msg = self._formatMessage(
                 msg,
-                f"Column {column} provided set was not in {', '.join(map(str, column_unique_list))}",
+                f"Column {column} provided set was not in actaul set of {', '.join(map(str, column_unique_list))}",
             )
             raise self.failureException(msg)
 
         return
 
-    expect_column_values_to_be_in_set = assertExpectColumnValuesToBeInSet
-
-    def assertExpectColumnValuesToBeOfType(
+    def expect_column_values_to_be_of_type(
         self, df, column: str, type_: Union[str, float, int], msg=""
     ):
         """Expect a column to contain values of a specified data type."""
 
         df = _get_dataframe_type(df)
 
+        self.extended = {
+            "id": 10,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "expected_type": str(type_),
+            },
+        }
+
         df_type = df[column].dtypes
         fstr = f"Column {column} was not type {type_}"
         if type_ is str:
             # Consider object a string
             if df_type.num != 17:
                 msg = self._formatMessage(msg, fstr)
+                self.extended["values"]["actual_type"] = "str"
                 raise self.failureException(msg)
 
         if type_ is int:
             if df_type.num not in [7, 9]:
                 msg = self._formatMessage(msg, fstr)
+                self.extended["values"]["actual_type"] = "int"
                 raise self.failureException(msg)
 
         if type_ is float:
             if df_type.num != 12:
                 msg = self._formatMessage(msg, fstr)
+                self.extended["values"]["actual_type"] = "float"
                 raise self.failureException(msg)
 
         if type_ not in [str, int, float]:
             msg = self._formatMessage(
                 msg, "Please check available types; str, float, int"
             )
+            self.extended["values"]["actual_type"] = "unknown"
             raise self.failureException(msg)
 
         return
 
-    expect_column_values_to_be_of_type = assertExpectColumnValuesToBeOfType
-
-    def assertExpectTableColumnsToMatchOrderedList(
+    def expect_table_columns_to_match_ordered_list(
         self, df, column_list: List[str], msg=""
     ):
         """
@@ -264,6 +567,11 @@ class GreatAssertions(unittest.TestCase):
 
         df = _get_dataframe_import_type(df)
 
+        self.extended = {
+            "id": 11,
+            "name": sys._getframe().f_code.co_name,
+        }
+
         if list(df.columns) != column_list:
             msg = self._formatMessage(
                 msg,
@@ -273,11 +581,7 @@ class GreatAssertions(unittest.TestCase):
 
         return
 
-    expect_table_columns_to_match_ordered_list = (
-        assertExpectTableColumnsToMatchOrderedList
-    )
-
-    def assertExpectTableColumnsToMatchSet(
+    def expect_table_columns_to_match_set(
         self, df, column_set: Optional[Union[Set[str], List[str]]], msg=""
     ):
         """Expect the columns to match a specified set.
@@ -291,6 +595,11 @@ class GreatAssertions(unittest.TestCase):
         """
         df = _get_dataframe_import_type(df)
 
+        self.extended = {
+            "id": 12,
+            "name": sys._getframe().f_code.co_name,
+        }
+
         column_set = set(column_set) if column_set is not None else set()
         if set(df.columns) != column_set:
             msg = self._formatMessage(
@@ -299,9 +608,7 @@ class GreatAssertions(unittest.TestCase):
             )
             raise self.failureException(msg)
 
-    expect_table_columns_to_match_set = assertExpectTableColumnsToMatchSet
-
-    def assertExpectDateRangeToBeLessThan(
+    def expect_date_range_to_be_less_than(
         self, df, column: str, date: str, date_format="%Y-%m-%d", msg=""
     ):
         """
@@ -321,6 +628,13 @@ class GreatAssertions(unittest.TestCase):
         """
 
         df = _get_dataframe_type(df)
+
+        self.extended = {
+            "id": 13,
+            "name": sys._getframe().f_code.co_name,
+            "values": {"expected_max_date": date},
+        }
+
         df[column] = df[column].apply(lambda dt: _default_null_dates(dt, date_format))
 
         results = df[df[column] >= datetime.strptime(date, date_format)]
@@ -331,13 +645,12 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Column {column} date is greater or equal than {date} found {dt}",
             )
+            self.extended["values"]["actual_max_date"] = str(dt)
             raise self.failureException(msg)
 
         return
 
-    expect_date_range_to_be_less_than = assertExpectDateRangeToBeLessThan
-
-    def assertExpectDateRangeToBeMoreThan(
+    def expect_date_range_to_be_more_than(
         self, df, column: str, date: str, date_format="%Y-%m-%d", msg=""
     ):
         """
@@ -357,6 +670,13 @@ class GreatAssertions(unittest.TestCase):
         """
 
         df = _get_dataframe_type(df)
+
+        self.extended = {
+            "id": 14,
+            "name": sys._getframe().f_code.co_name,
+            "values": {"expected_min_date": date},
+        }
+
         df[column] = df[column].apply(lambda dt: _default_null_dates(dt, date_format))
 
         results = df[df[column] <= datetime.strptime(date, date_format)]
@@ -367,13 +687,12 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Column {column} is less or equal than {date} found {dt}",
             )
+            self.extended["values"]["actual_min_date"] = str(dt)
             raise self.failureException(msg)
 
         return
 
-    assert_expect_date_range_to_be_more_than = assertExpectDateRangeToBeMoreThan
-
-    def assertExpectDateRangeToBeBetween(
+    def expect_date_range_to_be_between(
         self,
         df,
         column: str,
@@ -400,6 +719,16 @@ class GreatAssertions(unittest.TestCase):
         """
 
         df = _get_dataframe_type(df)
+
+        self.extended = {
+            "id": 15,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "expected_min_date": date_start,
+                "expected_max_date": date_end,
+            },
+        }
+
         df[column] = df[column].apply(lambda dt: _default_null_dates(dt, date_format))
 
         start_date = datetime.strptime(date_start, date_format)
@@ -421,13 +750,12 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Column {column} is not between {date_start} and {date_end} found {dt}",
             )
+            self.extended["values"]["first_found"] = str(dt)
             raise self.failureException(msg)
 
         return
 
-    expect_date_range_to_be_between = assertExpectDateRangeToBeBetween
-
-    def assertExpectColumnMeanToBeBetween(
+    def expect_column_mean_to_be_between(
         self, df, column, min_value: float, max_value: float, msg=""
     ):
         """
@@ -444,6 +772,16 @@ class GreatAssertions(unittest.TestCase):
 
         df = _get_dataframe_import_type(df)
 
+        self.extended = {
+            "id": 16,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "expected_min_value": min_value,
+                "expected_max_value": max_value,
+            },
+        }
+
         if min_value > max_value:
             msg = self._formatMessage(
                 msg,
@@ -457,6 +795,7 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Column {column} mean {format(mean_value, '.5f')} is less than min_value {min_value}",
             )
+            self.extended["values"]["actual_mean"] = mean_value
             raise self.failureException(msg)
 
         if max_value < mean_value:
@@ -464,13 +803,12 @@ class GreatAssertions(unittest.TestCase):
                 msg,
                 f"Column {column} mean {format(mean_value, '.5f')} is greater than max_value {max_value}",
             )
+            self.extended["values"]["actual_mean"] = mean_value
             raise self.failureException(msg)
 
         return
 
-    expect_column_mean_to_be_between = assertExpectColumnMeanToBeBetween
-
-    def assertExpectColumnValueCountsPercentToBeBetween(
+    def expect_column_value_counts_percent_to_be_between(
         self, df, column: str, value_counts: dict, msg=""
     ):
         """
@@ -494,12 +832,12 @@ class GreatAssertions(unittest.TestCase):
 
         value_counts = {"Y": {"min": 55, "max": 65}, "N": {"min": 35, "max": 45}}
 
-        self.assertExpectColumnValueCountsPercentToBeBetween(
+        self.expect_column_value_counts_percent_to_be_between(
                 df, "col_1", value_counts
         )
 
-        BlHAA
-        -----
+        Further
+        -------
 
         col_1 actual percentages: Y - 60%, N - 40%
 
@@ -508,6 +846,16 @@ class GreatAssertions(unittest.TestCase):
         """
 
         df = _get_dataframe_type(df)
+
+        self.extended = {
+            "id": 17,
+            "name": sys._getframe().f_code.co_name,
+            "values": {
+                "column": column,
+                "value_count": value_counts,
+            },
+        }
+
         result = df[column].value_counts()
 
         for key in value_counts:
@@ -516,9 +864,10 @@ class GreatAssertions(unittest.TestCase):
             try:
                 key_percent = (result[key] / len(df)) * 100
             except KeyError as e:
+                sorted_results = ", ".join(sorted(result.index.tolist()))
                 msg = self._formatMessage(
                     msg,
-                    f"Check the key {str(e)} is not in the available value counts names {', '.join(sorted(result.index.tolist()))}",
+                    f"Check the key {str(e)} is not in the available value counts names {sorted_results}",
                 )
                 raise self.failureException(msg)
 
@@ -536,19 +885,60 @@ class GreatAssertions(unittest.TestCase):
             if column_min > key_percent:
                 msg = self._formatMessage(
                     msg,
-                    f"Column {column} the actual value count of ({key}) is {format(key_percent, '.5f')}% is less than the min allowed of {value_counts[key]['min']}%",
+                    f"Column {column} the actual value count of ({key}) is {format(key_percent, '.5f')}% "
+                    f"is less than the min allowed of {value_counts[key]['min']}%",
                 )
+                self.extended["values"]["column_min"] = format(key_percent, ".5f")
+                self.extended["values"]["value_count_min"] = value_counts[key]["min"]
                 raise self.failureException(msg)
 
             if column_max < key_percent:
                 msg = self._formatMessage(
                     msg,
-                    f"Column {column} the actual value count of ({key}) is {format(key_percent, '.5f')}% is more than the max allowed of {value_counts[key]['max']}%",
+                    f"Column {column} the actual value count of ({key}) is {format(key_percent, '.5f')}% "
+                    f"is more than the max allowed of {value_counts[key]['max']}%",
                 )
+                self.extended["values"]["column_max"] = format(key_percent, ".5f")
+                self.extended["values"]["value_count_max"] = value_counts[key]["max"]
                 raise self.failureException(msg)
 
         return
 
-    expect_column_value_counts_percent_to_be_between = (
-        assertExpectColumnValueCountsPercentToBeBetween
-    )
+    def expect_frames_equal(
+        self, left, right, check_dtype=True, check_index=True, msg=""
+    ):
+        """
+        Expect two DataFrames to compare.
+
+        Parameters
+        ----------
+            left (DataFrame)   : Pandas or PySpark DataFrame
+            right (DataFrame)  : Pandas or PySpark DataFrame
+            check_dtype (bool) : Ignore Schema differences
+            check_index (bool) : Ignore index differences
+            msg (str)          : Optional message if the assertion fails
+
+        """
+        left = _get_dataframe_import_type(left)
+        right = _get_dataframe_import_type(right)
+
+        self.extended = {
+            "id": 18,
+            "name": sys._getframe().f_code.co_name,
+        }
+
+        if type(left) != type(right):
+            msg = self._formatMessage(
+                msg,
+                "Different DataFrame types",
+            )
+            raise self.failureException(msg)
+
+        try:
+            left.expect_frames_equal(right.df, check_dtype, check_index)
+        except AssertionError as e:
+            msg = self._formatMessage(
+                msg,
+                f"DataFrames are different with {e}",
+            )
+            raise self.failureException(msg)
